/************************************************************************/
/* Author: 	Hirotaka Miura.						*/
/* Contact: 	Hirotaka.Miura@gmail.com.				*/
/************************************************************************/

/************************************************************************/
/* 05/05/2011: 	Modified.						*/
/* 04/24/2011: 	Previouly modified.					*/
/* 04/22/2011: 	Created.						*/
/************************************************************************/

/************************************************************************/
/* Program: 	dijkstra_betweenness().					*/
/* Function: 	Betweenness centrality based on Dijkstra's SSSP algthm.	*/
/************************************************************************/
mata:
real vector dijkstra_betweenness(real matrix E, 
  |string scalar nodots){
  /* Declare variables.							*/
  struct matrix_struct scalar z /* Matrix return structure.		*/
  real matrix A /* Adjacency list matrix.				*/
  real matrix W /* Adjacency weight list matrix.			*/
  real matrix C /* Component matrix.					*/
  real matrix X /* Index matrix.					*/
  real matrix N /* Neighbor matrix holding labels and distances.	*/
  real matrix S /* Source-specific shortest path matrix.		*/
  real matrix B /* Betweenness score matrix.				*/
  real matrix T /* Shortest path tree matrix.				*/
  real vector b /* Betweenness centrality vector.			*/
  real vector e /* Vector of current estimates.				*/
  real vector c /* Vector to hold certain/uncertain.			*/
  real vector v /* Root vector.						*/
  real vector p /* Path vector.						*/
  real vector y /* Y-coordinates of sparse matrix.			*/
  real vector parent /* Parent node vector.				*/
  real vector child /* Child node vector.				*/
  real scalar i,j,k /* Row/column counters.				*/
  real scalar s /* Source vertex.					*/
  real scalar m /* Total number of nodes in network.			*/
  real scalar d /* Distance.						*/
  real scalar exit /* While loop exit indicator.			*/
  real scalar l1,l2,w1,w2 /* Temporary leaf and weight scores.		*/
  string scalar o /* Status display string.				*/
  /**********************************************************************/
  /* Begin program.							*/
  /**********************************************************************/
  /***** Implement component analysis to mark all disconnected nodes.	*/
  C=bfs_component(E,nodots)     
  /* Obtain adjacency list and adjacency weight list matrices.		*/
  z=adjacency_list(E); A=z.X1; W=z.X2  
  /* Obtain total number of nodes in network.				*/
  m=rows(A)    
  /* Inform user of program starting.					*/
  o="noisily display "+`""""'; stata(o); displayflush()
  o="Betweenness centrality calculation ("+strofreal(m)+" vertices)"
  o="noisily display "+`"""'+o+`"""'
  stata(o); displayflush()
  /* Display dots unless specified not to.				*/
  if(nodots==""){
    o="----+--- 1 ---+--- 2 ---+--- 3 ---+--- 4 ---+--- 5"
    o="noisily display "+`"""'+o+`"""'
    stata(o); displayflush() 
  }  
  /* Define betweenness score matrix.  Set initial values to ones as 	*/
  /* we'll mark non-leaf vertices with zero later.			*/
  B=J(m,m,1)
  /***** Loop through all nodes.					*/
  for(s=1;s<=m;s++){
    /***** Preliminary check: Continue if source has no neighbors.	*/
    if(nonmissing(A[s,.])==0){
      /* Display status unless specified not to.			*/
      if(nodots=="") display_status(m,s)    
      continue
    }    
    /* Define vector to hold path numbers. Set path # for source to one.*/
    p=J(1,m,.); p[s]=1
    /* Define vector to hold current estimates.  			*/			
    e=J(1,m,.)
    /* Define vector to note whether we're certain or not.		*/
    c=C[s,.]; _editvalue(c,1,.); _editvalue(c,0,1); _editmissing(c,0)
    /* Mark distance as being certain.					*/
    e[s]=0; c[s]=1   
    /***** Obtain list of neighbors of vertex s.			*/
    /* No need to add distance - distance of s is zero.			*/
    N=(A[s,.]\W[s,.]); N=N[.,(1..nonmissing(N)/2)]
    /***** Initialize index matrix with nbhrs of s sorted by distance.	*/
    X=sort(N',2)
    /***** Record distance estimates.					*/
    for(j=1;j<=cols(N);j++){
      /* Record neighbors' distance estimates. Since all entries missing*/
      /* except for source vertex entry, just straight input.		*/
      e[N[1,j]]=N[2,j]      
      /* Set path number.						*/
      p[N[1,j]]=1
      /* Set shortest path tree array entry.				*/
      if(j==1) S=(N[1,j],s)
      else S=(S\(N[1,j],s))
    }   
    /* Obtain current root information.					*/
    v=X[1,.]
    /* Mark distance as being certain.					*/
    c[v[1]]=1   
    /***** Loop while certain/uncertain vector is unfilled.		*/
    /* Set exit indicator.						*/
    exit=0
    while(exit!=1){       
      /***** Set exit indicator to exit if appropriate condition met.	*/
      if(sum(c)==(cols(c))) exit=1
      /***** Add neighbors if v has neighbors.  It may be the case that	*/
      /* X might have rows > 1 but v may not have any neighbors.	*/
      if(nonmissing(A[v[1],.])>0){
	/* Obtain list of neighbors of vertex v.			*/
	N=(A[v[1],.]\W[v[1],.]); N=N[.,(1..nonmissing(N)/2)]    
	/* Add distance so far from s to v.				*/
	N[2,.]=N[2,.]:+v[2]    
	/* Loop through neighbors and append to X if appropriate.	*/
	for(j=1;j<=cols(N);j++) dijkstra_betweenness_insert(X,N[.,j],e,p,v,S)
      }       
      /***** Obtain new root.						*/
      /* Remove root node if index matrix has more than one entry.	*/
      if(rows(X)>1) X=X[(2..rows(X)),.]      
      /* Sort index matrix.						*/
      _sort(X,2)
      /* Set new root and update certain/uncertain vector.		*/
      v=X[1,.]; c[v[1]]=1
    }
    /********************************************************************/
    /***** Betweenness centrality calculation.				*/
    /********************************************************************/
    /* Construct source specific shortest path tree matrix.		*/
    y=uniqrows(S[.,2])
    /***** Initialize betweenness matrix.				*/
    /* Mark non-leaf nodes.  Mat B initialized with all ones.		*/
    for(i=1;i<=rows(y);i++) B[s,y[i]]=. 
    /***** Generate tree matrix.					*/
    T=dijkstra_betweenness_spt(S,s)
    /* Obtain fathest distance.						*/
    d=max(T[.,2])-1
    /* Loop until source vertex is reached.				*/
    while(d>-1){
      parent=select(T[.,1],T[.,2]:==d)'
      for(j=1;j<=cols(parent);j++){
	/* Obtain child node(s).					*/
	child=select(S[.,1],S[.,2]:==parent[j])'
	if(cols(child)==0) continue
	/* Set initial parent betweenness score to one.			*/
	B[s,parent[j]]=1
	/* Loop through child node(s).					*/
	for(k=1;k<=cols(child);k++){
	  /* Assign temporary leaf and weight scores.			*/
	  l1=B[s,parent[j]]
	  l2=B[s,child[k]]
	  w1=p[parent[j]]
	  w2=p[child[k]]
	  /* Re-assign leaf score if all values present.		*/
	  if(l2!=.&w1!=.&w2!=.&w2!=0) B[s,parent[j]]=l1+l2*w1/w2      
	}      
      }
      --d
    }
    /* Display status unless specified not to.				*/
    if(nodots=="") display_status(m,s)    
  }
  /* Adjust betweenness score matrix and return betweenness vector.	*/
  B=B:-1
  b=colsum(B)'
  /* Notify program is over.						*/
  o="Betweenness centrality calculation completed"
  o="noisily display "+`"""'+o+`"""'
  stata(o); displayflush()   
  return(b)
}  
/************************************************************************/
/* Program: dijkstra_betweenness_spt().					*/ 
/* Function: Generates tree matrix containing directed shortest paths.	*/
/* Notes: If two distances available for a node, farthest is recorded.	*/
/************************************************************************/
real matrix dijkstra_betweenness_spt(real matrix E, real scalar s){
  /* Declare variables.							*/
  real matrix T /* Tree matrix.						*/
  real vector c /* Child node(s) vector.				*/
  real vector p /* Parent node(s) vector.				*/
  real scalar j,k /* Column counters.					*/
  real scalar i /* Distance counter.					*/
  real scalar exit /* While loop exit indicator.			*/
  /* Begin program.							*/
  /***** Generate tree matrix.						*/
  /* Set up for while loop to go from top to bottom.			*/
  /* Obtain child node(s) of source vertex.				*/
  c=select(E[.,1],E[.,2]:==s)'
  //c=select(1..cols(X),X[s,.]:!=.)
  /* Input child node(s) into tree matrix with distance one.		*/
  T=(c',J(cols(c),1,1))
  i=2
  exit=0
  /* Loop while bottom is not reached.					*/
  while(exit!=1){
    p=c
    c=J(1,0,.)
    for(j=1;j<=cols(p);j++){
      if(cols(select(E[.,1],E[.,2]:==p[j])')!=0){
        c=(c,select(E[.,1],E[.,2]:==p[j])')
      }
    }
    /* Exit if no more child node(s).					*/
    if(cols(c)==0) exit=1
    c=uniqrows(c')'
    /* Remove shorter distance if node already in tree.			*/
    for(k=1;k<=cols(c);k++){
      if(anyof(T[.,1],c[k])) T=select(T,T[.,1]:!=c[k])
    }
    T=(T\(c',J(cols(c),1,i)))
    ++i
  }    
  /* Return tree matrix.						*/
  return(T)
}
/************************************************************************/
/* Program: insert().							*/
/* Function: Carry out appropriate insertion of newly discovered nodes.	*/
/************************************************************************/
void dijkstra_betweenness_insert(real matrix X, real vector n, 
  real vector e, real vector p, real vector v, real matrix E){
  /* Declare variables.							*/  
  /* Begin program.							*/
  /**********************************************************************/
  /* Case: Incoming node is already in index matrix.			*/
  /**********************************************************************/
  /* Nodes are in the index matrix only if their estimated distance	*/
  /* has been recorded in the estimates vector.				*/
  if(anyof(X[.,1],n[1])){
    /* If current estimate is larger than new estimate, update		*/
    /* index matrix, and estimates vector.				*/
    if(e[n[1]]>n[2]){
      /* First update estimates vector.					*/
      e[n[1]]=n[2]
      /* Second update index matrix by first removing current entry	*/
      /* and then appending new entry.					*/
      X=select(X,X[.,1]:!=n[1])
      X=(X\n')
      /* Re-start path count.						*/
      p[n[1]]=p[v[1]]
      /* Point shortest path edge to new vertex.			*/
      E=select(E,E[.,1]:!=n[1])
      if(rows(E)==0) E=(n[1],v[1])
      else E=uniqrows(E\(n[1],v[1]))
    }
    /* Increment path number if estimated distance is the same.		*/
    else if(e[n[1]]==n[2]){
      p[n[1]]=p[n[1]]+p[v[1]] 
      /* Update shortest path edge matrix.				*/
      E=uniqrows(E\(n[1],v[1]))
    }
    /* Exit function if new estimate is greater than current estimate.	*/
    else return
  }    
  /**********************************************************************/
  /* Case: Incoming node is not in the index matrix.			*/
  /**********************************************************************/  
  else if(anyof(X[.,1],n[1])==0){
    /* If estimates already defined in the estimates vector, then the	*/
    /* node has already been in the index matrix and has been removed 	*/
    /* with its estimate being noted as certain. Therefore increment 	*/
    /* path number if new estimate is equal to current estimate then	*/
    /* exit function or otherwise just exit.				*/
    if(e[n[1]]!=.){
      if(e[n[1]]==n[2]){
        p[n[1]]=p[n[1]]+p[v[1]]
        /* Update shortest path edge matrix.				*/
        E=uniqrows(E\(n[1],v[1]))
      }
      else return
    }
    /* If estimates are not yet defined, record new estimate, and insert*/
    /* new node into index matrix.					*/
    else if(e[n[1]]==.){
      /* Insert estimated distance into estimates vector.		*/
      e[n[1]]=n[2]
      /* Append to index matrix.					*/
      X=(X\n')
      /* Carry over shortest path count from parent node.		*/
      p[n[1]]=p[v[1]]
      /* Fill in shortest path matrix.					*/
      E=uniqrows(E\(n[1],v[1]))
    }  
  }
}     
end



