/************************************************************************/
/* Author: 	Hirotaka Miura.						*/
/* Contact: 	Hirotaka.Miura@gmail.com.				*/
/************************************************************************/

/************************************************************************/
/* 05/05/2011: 	Modified.						*/
/* 04/24/2011: 	Previouly modified.					*/
/* 04/22/2011: 	Created.						*/
/************************************************************************/

/************************************************************************/
/* Program: 	dijkstra_sp().						*/
/* Function: 	Dijkstra's single-source shortest-path algorithm.	*/
/************************************************************************/
mata:
struct matrix_struct scalar dijkstra_sp(real matrix E, 
  | string scalar nodots){
  /* Declare variables.							*/
  struct matrix_struct scalar z /* Return structure.			*/
  real matrix C /* Component matrix.					*/
  real matrix A /* Adjacency list matrix.				*/
  real matrix W /* Adjacency weight list matrix.			*/
  real matrix D /* Distance matrix.					*/
  real matrix P /* Path matrix.						*/
  real matrix X /* Index matrix.					*/
  real matrix N /* Neighbor matrix holding labels and distances.	*/  
  real vector e /* Vector of current estimates.				*/
  real vector c /* Vector to hold certain/uncertain.			*/
  real vector v /* Root vector.						*/
  real vector p /* Path vector.						*/
  real vector t /* Source-specific diconnected nodes vector.		*/
  real scalar j /* Column counter.					*/
  real scalar s /* Source vertex.					*/
  real scalar m /* Total number of nodes in network.			*/
  real scalar exit /* While loop exit indicator.			*/  
  string scalar o /* Status display string.				*/  
  /**********************************************************************/
  /* Begin program.							*/
  /**********************************************************************/
  /***** Implement component analysis to mark all disconnected nodes.	*/
  C=bfs_component(E,nodots)     
  /* Obtain adjacency list and adjacency weight list matrices.		*/
  z=adjacency_list(E); A=z.X1; W=z.X2  
  /* Obtain total number of nodes in network.				*/
  m=rows(A)    
  /* Inform user of program starting.					*/
  o="noisily display "+`""""'; stata(o); displayflush()
  o="Dijkstra's search algorithm ("+strofreal(m)+" vertices)"
  o="noisily display "+`"""'+o+`"""'
  stata(o); displayflush()
  /* Display dots unless specified not to.				*/
  if(nodots==""){
    o="----+--- 1 ---+--- 2 ---+--- 3 ---+--- 4 ---+--- 5"
    o="noisily display "+`"""'+o+`"""'
    stata(o); displayflush() 
  }    
  /* Define distance and path matrices.					*/
  D=J(m,m,.); P=J(m,m,.)  
  /***** Loop through all nodes.					*/
  for(s=1;s<=m;s++){
    /* Define vector hold path numbers.  Set path # for source to one.	*/
    p=J(1,m,.); p[s]=1
    /* Define vector to hold current estimates.  			*/			
    e=J(1,m,.)
    /* Define vector to note whether we're certain or not.		*/
    c=J(1,m,0)
    /* Mark distance as being certain.					*/
    e[s]=0; c[s]=1   
    /***** Preliminary check: Continue if source has no neighbors.	*/
    if(nonmissing(A[s,.])==0){
      D[s,.]=e
      P[s,.]=p
      /* Display status unless specified not to.			*/
      if(nodots=="") display_status(m,s)    
      continue
    }
    /***** Mark all disconnected nodes.					*/
    t=select(1..cols(C),C[s,.]:==0)
    for(j=1;j<=cols(t);j++) c[t[j]]=1
    /***** Obtain list of neighbors of vertex s.			*/
    /* No need to add distance - distance of s is zero.			*/
    N=(A[s,.]\W[s,.]); N=N[.,(1..nonmissing(N)/2)]
    /***** Initialize index matrix with nbhrs of s sorted by distance.	*/
    X=sort(N',2)
    /***** Record distance estimates.					*/
    for(j=1;j<=cols(N);j++){
      /* Record neighbors' distance estimates. Since all entries missing*/
      /* except for source vertex entry, just straight input.		*/
      e[N[1,j]]=N[2,j]      
      /* Set path number.						*/
      p[N[1,j]]=1
    }   
    /* Obtain current root information.					*/
    v=X[1,.]
    /* Mark distance as being certain.					*/
    c[v[1]]=1   
    /***** Loop while certain/uncertain vector is unfilled.		*/
    /* Set exit indicator.						*/
    exit=0    
    while(exit!=1){       
      /***** Set exit indicator to exit if appropriate condition met.	*/
      if(sum(c)==(cols(c))) exit=1
      /***** Add neighbors if v has neighbors.  It may be the case that	*/
      /* X might have rows > 1 but v may not have any neighbors.	*/
      if(nonmissing(A[v[1],.])>0){
	/* Obtain list of neighbors of vertex v.			*/
	N=(A[v[1],.]\W[v[1],.]); N=N[.,(1..nonmissing(N)/2)]   
	/* Add distance so far from s to v.				*/
	N[2,.]=N[2,.]:+v[2]    
	/* Loop through neighbors and append to X if appropriate.	*/
	for(j=1;j<=cols(N);j++) dijkstra_insert(X,N[.,j],e,p,v)
      }       
      /***** Obtain new root.						*/
      /* Remove root node if index matrix has more than one entry.	*/
      if(rows(X)>1) X=X[(2..rows(X)),.]      
      /* Sort index matrix.						*/
      _sort(X,2)
      /* Set new root and update certain/uncertain vector.		*/
      v=X[1,.]; c[v[1]]=1
    }
    D[s,.]=e
    P[s,.]=p
    /* Display status unless specified not to.				*/
    if(nodots=="") display_status(m,s)    
  }
  /* Notify program is over.						*/
  o="Dijkstra's search algorithm completed"
  o="noisily display "+`"""'+o+`"""'
  stata(o); displayflush() 
  /* Return distance and path matrices.					*/
  z.X1=D; z.X2=P
  return(z)
}
/************************************************************************/
/* Program: dijkstra_insert().						*/
/* Function: Carry out appropriate insertion of newly discovered nodes.	*/
/************************************************************************/
void dijkstra_insert(real matrix X, real vector n, 
  real vector e, real vector p, real vector v){
  /* Declare variables.							*/  
  /* Begin program.							*/
  /**********************************************************************/
  /* Case: Incoming node is already in index matrix.			*/
  /**********************************************************************/
  /* Nodes are in the index matrix only if their estimated distance	*/
  /* has been recorded in the estimates vector.				*/
  if(anyof(X[.,1],n[1])){
    /* If current estimate is larger than new estimate, update		*/
    /* index matrix, and estimates vector.				*/
    if(e[n[1]]>n[2]){
      /* First update estimates vector.					*/
      e[n[1]]=n[2]
      /* Second update index matrix by first removing current entry	*/
      /* and then appending new entry.					*/
      X=select(X,X[.,1]:!=n[1])
      X=(X\n')
      /* Re-start path count.						*/
      p[n[1]]=p[v[1]]
    }
    /* Increment path number if estimated distance is the same.		*/
    else if(e[n[1]]==n[2]){
      p[n[1]]=p[n[1]]+p[v[1]] 
    }
    /* Exit function if new estimate is greater than current estimate.	*/
    else return
  }    
  /**********************************************************************/
  /* Case: Incoming node is not in the index matrix.			*/
  /**********************************************************************/  
  else if(anyof(X[.,1],n[1])==0){
    /* If estimates already defined in the estimates vector, then the	*/
    /* node has already been in the index matrix and has been removed 	*/
    /* with its estimate being noted as certain. Therefore increment 	*/
    /* path number if new estimate is equal to current estimate then	*/
    /* exit function or otherwise just exit.				*/
    if(e[n[1]]!=.){
      if(e[n[1]]==n[2]){
        p[n[1]]=p[n[1]]+p[v[1]]
      }
      else return
    }
    /* If estimates are not yet defined, record new estimate, and insert*/
    /* new node into index matrix.					*/
    else if(e[n[1]]==.){
      /* Insert estimated distance into estimates vector.		*/
      e[n[1]]=n[2]
      /* Append to index matrix.					*/
      X=(X\n')
      /* Carry over shortest path count from parent node.		*/
      p[n[1]]=p[v[1]]
    }  
  }
} 
end




